@@ -1,7 +1,7 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2016 Magnus Edenhill
+ * Copyright (c) 2016-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -98,6 +98,7 @@ static RD_UNUSED RD_INLINE int rd_kafka_event_setup(rd_kafka_t *rk,
         case RD_KAFKA_EVENT_DELETETOPICS_RESULT:
         case RD_KAFKA_EVENT_CREATEPARTITIONS_RESULT:
         case RD_KAFKA_EVENT_ALTERCONFIGS_RESULT:
+        case RD_KAFKA_EVENT_INCREMENTALALTERCONFIGS_RESULT:
         case RD_KAFKA_EVENT_DESCRIBECONFIGS_RESULT:
         case RD_KAFKA_EVENT_DELETERECORDS_RESULT:
         case RD_KAFKA_EVENT_LISTCONSUMERGROUPS_RESULT:
@@ -110,6 +111,8 @@ static RD_UNUSED RD_INLINE int rd_kafka_event_setup(rd_kafka_t *rk,
         case RD_KAFKA_EVENT_ALTERCONSUMERGROUPOFFSETS_RESULT:
         case RD_KAFKA_EVENT_LISTCONSUMERGROUPOFFSETS_RESULT:
         case RD_KAFKA_EVENT_OAUTHBEARER_TOKEN_REFRESH:
+        case RD_KAFKA_EVENT_DESCRIBEUSERSCRAMCREDENTIALS_RESULT:
+        case RD_KAFKA_EVENT_ALTERUSERSCRAMCREDENTIALS_RESULT:
                 return 1;
 
         default:
