@@ -1,7 +1,8 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2012-2013, Magnus Edenhill
+ * Copyright (c) 2012-2022, Magnus Edenhill
+ *               2023, Confluent Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -76,9 +77,9 @@
  * @locks rd_kafka_wrlock()
  */
 static RD_INLINE void
-rd_kafka_metadata_cache_delete (rd_kafka_t *rk,
-                                struct rd_kafka_metadata_cache_entry *rkmce,
-                                int unlink_avl) {
+rd_kafka_metadata_cache_delete(rd_kafka_t *rk,
+                               struct rd_kafka_metadata_cache_entry *rkmce,
+                               int unlink_avl) {
         if (unlink_avl)
                 RD_AVL_REMOVE_ELM(&rk->rk_metadata_cache.rkmc_avl, rkmce);
         TAILQ_REMOVE(&rk->rk_metadata_cache.rkmc_expiry, rkmce, rkmce_link);
@@ -93,8 +94,8 @@ rd_kafka_metadata_cache_delete (rd_kafka_t *rk,
  * @locks rd_kafka_wrlock()
  * @returns 1 if entry was found and removed, else 0.
  */
-static int rd_kafka_metadata_cache_delete_by_name (rd_kafka_t *rk,
-                                                    const char *topic) {
+static int rd_kafka_metadata_cache_delete_by_name(rd_kafka_t *rk,
+                                                  const char *topic) {
         struct rd_kafka_metadata_cache_entry *rkmce;
 
         rkmce = rd_kafka_metadata_cache_find(rk, topic, 1);
@@ -103,15 +104,15 @@ static int rd_kafka_metadata_cache_delete_by_name (rd_kafka_t *rk,
         return rkmce ? 1 : 0;
 }
 
-static int rd_kafka_metadata_cache_evict (rd_kafka_t *rk);
+static int rd_kafka_metadata_cache_evict(rd_kafka_t *rk);
 
 /**
  * @brief Cache eviction timer callback.
  * @locality rdkafka main thread
  * @locks NOT rd_kafka_*lock()
  */
-static void rd_kafka_metadata_cache_evict_tmr_cb (rd_kafka_timers_t *rkts,
-                                                  void *arg) {
+static void rd_kafka_metadata_cache_evict_tmr_cb(rd_kafka_timers_t *rkts,
+                                                 void *arg) {
         rd_kafka_t *rk = arg;
 
         rd_kafka_wrlock(rk);
@@ -126,10 +127,10 @@ static void rd_kafka_metadata_cache_evict_tmr_cb (rd_kafka_timers_t *rkts,
  *
  * @returns the number of entries evicted.
  *
- * @locks rd_kafka_wrlock()
+ * @locks_required rd_kafka_wrlock()
  */
-static int rd_kafka_metadata_cache_evict (rd_kafka_t *rk) {
-        int cnt = 0;
+static int rd_kafka_metadata_cache_evict(rd_kafka_t *rk) {
+        int cnt     = 0;
         rd_ts_t now = rd_clock();
         struct rd_kafka_metadata_cache_entry *rkmce;
 
@@ -143,8 +144,7 @@ static int rd_kafka_metadata_cache_evict (rd_kafka_t *rk) {
                 rd_kafka_timer_start(&rk->rk_timers,
                                      &rk->rk_metadata_cache.rkmc_expiry_tmr,
                                      rkmce->rkmce_ts_expires - now,
-                                     rd_kafka_metadata_cache_evict_tmr_cb,
-                                     rk);
+                                     rd_kafka_metadata_cache_evict_tmr_cb, rk);
         else
                 rd_kafka_timer_stop(&rk->rk_timers,
                                     &rk->rk_metadata_cache.rkmc_expiry_tmr, 1);
@@ -161,6 +161,50 @@ static int rd_kafka_metadata_cache_evict (rd_kafka_t *rk) {
 }
 
 
+/**
+ * @brief Evict timed out entries from cache based on their insert/update time
+ *        rather than expiry time. Any entries older than \p ts will be evicted.
+ *
+ * @returns the number of entries evicted.
+ *
+ * @locks_required rd_kafka_wrlock()
+ */
+int rd_kafka_metadata_cache_evict_by_age(rd_kafka_t *rk, rd_ts_t ts) {
+        int cnt = 0;
+        struct rd_kafka_metadata_cache_entry *rkmce, *tmp;
+
+        TAILQ_FOREACH_SAFE(rkmce, &rk->rk_metadata_cache.rkmc_expiry,
+                           rkmce_link, tmp) {
+                if (rkmce->rkmce_ts_insert <= ts) {
+                        rd_kafka_metadata_cache_delete(rk, rkmce, 1);
+                        cnt++;
+                }
+        }
+
+        /* Update expiry timer */
+        rkmce = TAILQ_FIRST(&rk->rk_metadata_cache.rkmc_expiry);
+        if (rkmce)
+                rd_kafka_timer_start(&rk->rk_timers,
+                                     &rk->rk_metadata_cache.rkmc_expiry_tmr,
+                                     rkmce->rkmce_ts_expires - rd_clock(),
+                                     rd_kafka_metadata_cache_evict_tmr_cb, rk);
+        else
+                rd_kafka_timer_stop(&rk->rk_timers,
+                                    &rk->rk_metadata_cache.rkmc_expiry_tmr, 1);
+
+        rd_kafka_dbg(rk, METADATA, "METADATA",
+                     "Expired %d entries older than %dms from metadata cache "
+                     "(%d entries remain)",
+                     cnt, (int)((rd_clock() - ts) / 1000),
+                     rk->rk_metadata_cache.rkmc_cnt);
+
+        if (cnt)
+                rd_kafka_metadata_cache_propagate_changes(rk);
+
+        return cnt;
+}
+
+
 /**
  * @brief Find cache entry by topic name
  *
@@ -169,7 +213,7 @@ static int rd_kafka_metadata_cache_evict (rd_kafka_t *rk) {
  * @locks rd_kafka_*lock()
  */
 struct rd_kafka_metadata_cache_entry *
-rd_kafka_metadata_cache_find (rd_kafka_t *rk, const char *topic, int valid) {
+rd_kafka_metadata_cache_find(rd_kafka_t *rk, const char *topic, int valid) {
         struct rd_kafka_metadata_cache_entry skel, *rkmce;
         skel.rkmce_mtopic.topic = (char *)topic;
         rkmce = RD_AVL_FIND(&rk->rk_metadata_cache.rkmc_avl, &skel);
@@ -182,8 +226,7 @@ rd_kafka_metadata_cache_find (rd_kafka_t *rk, const char *topic, int valid) {
 /**
  * @brief Partition (id) comparator
  */
-int rd_kafka_metadata_partition_id_cmp (const void *_a,
-                                        const void *_b) {
+int rd_kafka_metadata_partition_id_cmp(const void *_a, const void *_b) {
         const rd_kafka_metadata_partition_t *a = _a, *b = _b;
         return RD_CMP(a->id, b->id);
 }
@@ -196,12 +239,18 @@ int rd_kafka_metadata_partition_id_cmp (const void *_a,
  *
  * @locks_required rd_kafka_wrlock()
  */
-static struct rd_kafka_metadata_cache_entry *
-rd_kafka_metadata_cache_insert (rd_kafka_t *rk,
-                                const rd_kafka_metadata_topic_t *mtopic,
-                                rd_ts_t now, rd_ts_t ts_expires) {
+static struct rd_kafka_metadata_cache_entry *rd_kafka_metadata_cache_insert(
+    rd_kafka_t *rk,
+    const rd_kafka_metadata_topic_t *mtopic,
+    const rd_kafka_metadata_topic_internal_t *metadata_internal_topic,
+    rd_ts_t now,
+    rd_ts_t ts_expires,
+    rd_bool_t include_racks,
+    rd_kafka_metadata_broker_internal_t *brokers_internal,
+    size_t broker_cnt) {
         struct rd_kafka_metadata_cache_entry *rkmce, *old;
         size_t topic_len;
+        size_t racks_size = 0;
         rd_tmpabuf_t tbuf;
         int i;
 
@@ -210,47 +259,98 @@ rd_kafka_metadata_cache_insert (rd_kafka_t *rk,
          * rd_tmpabuf_t provides the infrastructure to do this.
          * Because of this we copy all the structs verbatim but
          * any pointer fields needs to be copied explicitly to update
-         * the pointer address. */
+         * the pointer address.
+         * See also rd_kafka_metadata_cache_delete which frees this. */
         topic_len = strlen(mtopic->topic) + 1;
-        rd_tmpabuf_new(&tbuf,
-                       RD_ROUNDUP(sizeof(*rkmce), 8) +
-                       RD_ROUNDUP(topic_len, 8) +
-                       (mtopic->partition_cnt *
-                        RD_ROUNDUP(sizeof(*mtopic->partitions), 8)),
-                       1/*assert on fail*/);
+
+        for (i = 0; include_racks && i < mtopic->partition_cnt; i++) {
+                size_t j;
+                racks_size += RD_ROUNDUP(
+                    metadata_internal_topic->partitions[i].racks_cnt *
+                        sizeof(char *),
+                    8);
+                for (j = 0;
+                     j < metadata_internal_topic->partitions[i].racks_cnt;
+                     j++) {
+                        racks_size += RD_ROUNDUP(
+                            strlen(metadata_internal_topic->partitions[i]
+                                       .racks[j]) +
+                                1,
+                            8);
+                }
+        }
+
+        rd_tmpabuf_new(
+            &tbuf,
+            RD_ROUNDUP(sizeof(*rkmce), 8) + RD_ROUNDUP(topic_len, 8) +
+                (mtopic->partition_cnt *
+                 RD_ROUNDUP(sizeof(*mtopic->partitions), 8)) +
+                (mtopic->partition_cnt *
+                 RD_ROUNDUP(sizeof(*metadata_internal_topic->partitions), 8)) +
+                racks_size,
+            1 /*assert on fail*/);
 
         rkmce = rd_tmpabuf_alloc(&tbuf, sizeof(*rkmce));
 
         rkmce->rkmce_mtopic = *mtopic;
 
+        rkmce->rkmce_metadata_internal_topic = *metadata_internal_topic;
+
         /* Copy topic name and update pointer */
         rkmce->rkmce_mtopic.topic = rd_tmpabuf_write_str(&tbuf, mtopic->topic);
 
         /* Copy partition array and update pointer */
-        rkmce->rkmce_mtopic.partitions =
-                rd_tmpabuf_write(&tbuf, mtopic->partitions,
-                                 mtopic->partition_cnt *
-                                 sizeof(*mtopic->partitions));
+        rkmce->rkmce_mtopic.partitions = rd_tmpabuf_write(
+            &tbuf, mtopic->partitions,
+            mtopic->partition_cnt * sizeof(*mtopic->partitions));
+
+        /* Copy partition array (internal) and update pointer */
+        rkmce->rkmce_metadata_internal_topic.partitions =
+            rd_tmpabuf_write(&tbuf, metadata_internal_topic->partitions,
+                             mtopic->partition_cnt *
+                                 sizeof(*metadata_internal_topic->partitions));
 
-        /* Clear uncached fields. */
-        for (i = 0 ; i < mtopic->partition_cnt ; i++) {
-                rkmce->rkmce_mtopic.partitions[i].replicas = NULL;
-                rkmce->rkmce_mtopic.partitions[i].replica_cnt = 0;
-                rkmce->rkmce_mtopic.partitions[i].isrs = NULL;
-                rkmce->rkmce_mtopic.partitions[i].isr_cnt = 0;
-        }
 
         /* Sort partitions for future bsearch() lookups. */
-        qsort(rkmce->rkmce_mtopic.partitions,
-              rkmce->rkmce_mtopic.partition_cnt,
+        qsort(rkmce->rkmce_mtopic.partitions, rkmce->rkmce_mtopic.partition_cnt,
               sizeof(*rkmce->rkmce_mtopic.partitions),
               rd_kafka_metadata_partition_id_cmp);
 
-        TAILQ_INSERT_TAIL(&rk->rk_metadata_cache.rkmc_expiry,
-                          rkmce, rkmce_link);
+        /* partitions (internal) are already sorted. */
+
+        if (include_racks) {
+                for (i = 0; i < rkmce->rkmce_mtopic.partition_cnt; i++) {
+                        size_t j;
+                        rd_kafka_metadata_partition_t *mdp =
+                            &rkmce->rkmce_mtopic.partitions[i];
+                        rd_kafka_metadata_partition_internal_t *mdpi =
+                            &rkmce->rkmce_metadata_internal_topic.partitions[i];
+                        rd_kafka_metadata_partition_internal_t *mdpi_orig =
+                            &metadata_internal_topic->partitions[i];
+
+                        if (mdp->replica_cnt == 0 || mdpi->racks_cnt == 0)
+                                continue;
+
+                        mdpi->racks = rd_tmpabuf_alloc(
+                            &tbuf, sizeof(char *) * mdpi->racks_cnt);
+                        for (j = 0; j < mdpi_orig->racks_cnt; j++)
+                                mdpi->racks[j] = rd_tmpabuf_write_str(
+                                    &tbuf, mdpi_orig->racks[j]);
+                }
+        }
+
+        /* Clear uncached fields. */
+        for (i = 0; i < mtopic->partition_cnt; i++) {
+                rkmce->rkmce_mtopic.partitions[i].replicas    = NULL;
+                rkmce->rkmce_mtopic.partitions[i].replica_cnt = 0;
+                rkmce->rkmce_mtopic.partitions[i].isrs        = NULL;
+                rkmce->rkmce_mtopic.partitions[i].isr_cnt     = 0;
+        }
+        TAILQ_INSERT_TAIL(&rk->rk_metadata_cache.rkmc_expiry, rkmce,
+                          rkmce_link);
         rk->rk_metadata_cache.rkmc_cnt++;
         rkmce->rkmce_ts_expires = ts_expires;
-        rkmce->rkmce_ts_insert = now;
+        rkmce->rkmce_ts_insert  = now;
 
         /* Insert (and replace existing) entry. */
         old = RD_AVL_INSERT(&rk->rk_metadata_cache.rkmc_avl, rkmce,
@@ -269,7 +369,7 @@ rd_kafka_metadata_cache_insert (rd_kafka_t *rk,
  *
  * @locks_required rd_kafka_wrlock()
  */
-void rd_kafka_metadata_cache_purge (rd_kafka_t *rk, rd_bool_t purge_observers) {
+void rd_kafka_metadata_cache_purge(rd_kafka_t *rk, rd_bool_t purge_observers) {
         struct rd_kafka_metadata_cache_entry *rkmce;
         int was_empty = TAILQ_EMPTY(&rk->rk_metadata_cache.rkmc_expiry);
 
@@ -293,15 +393,14 @@ void rd_kafka_metadata_cache_purge (rd_kafka_t *rk, rd_bool_t purge_observers) {
  *
  * @locks rd_kafka_wrlock()
  */
-void rd_kafka_metadata_cache_expiry_start (rd_kafka_t *rk) {
+void rd_kafka_metadata_cache_expiry_start(rd_kafka_t *rk) {
         struct rd_kafka_metadata_cache_entry *rkmce;
 
         if ((rkmce = TAILQ_FIRST(&rk->rk_metadata_cache.rkmc_expiry)))
                 rd_kafka_timer_start(&rk->rk_timers,
                                      &rk->rk_metadata_cache.rkmc_expiry_tmr,
                                      rkmce->rkmce_ts_expires - rd_clock(),
-                                     rd_kafka_metadata_cache_evict_tmr_cb,
-                                     rk);
+                                     rd_kafka_metadata_cache_evict_tmr_cb, rk);
 }
 
 /**
@@ -325,13 +424,17 @@ void rd_kafka_metadata_cache_expiry_start (rd_kafka_t *rk) {
  *
  * @locks rd_kafka_wrlock()
  */
-void
-rd_kafka_metadata_cache_topic_update (rd_kafka_t *rk,
-                                      const rd_kafka_metadata_topic_t *mdt,
-                                      rd_bool_t propagate) {
-        rd_ts_t now = rd_clock();
+void rd_kafka_metadata_cache_topic_update(
+    rd_kafka_t *rk,
+    const rd_kafka_metadata_topic_t *mdt,
+    const rd_kafka_metadata_topic_internal_t *mdit,
+    rd_bool_t propagate,
+    rd_bool_t include_racks,
+    rd_kafka_metadata_broker_internal_t *brokers,
+    size_t broker_cnt) {
+        rd_ts_t now        = rd_clock();
         rd_ts_t ts_expires = now + (rk->rk_conf.metadata_max_age_ms * 1000);
-        int changed = 1;
+        int changed        = 1;
 
         /* Cache unknown topics for a short while (100ms) to allow the cgrp
          * logic to find negative cache hits. */
@@ -341,57 +444,18 @@ rd_kafka_metadata_cache_topic_update (rd_kafka_t *rk,
         if (!mdt->err ||
             mdt->err == RD_KAFKA_RESP_ERR_TOPIC_AUTHORIZATION_FAILED ||
             mdt->err == RD_KAFKA_RESP_ERR_UNKNOWN_TOPIC_OR_PART)
-                rd_kafka_metadata_cache_insert(rk, mdt, now, ts_expires);
+                rd_kafka_metadata_cache_insert(rk, mdt, mdit, now, ts_expires,
+                                               include_racks, brokers,
+                                               broker_cnt);
         else
-                changed = rd_kafka_metadata_cache_delete_by_name(rk,
-                                                                 mdt->topic);
+                changed =
+                    rd_kafka_metadata_cache_delete_by_name(rk, mdt->topic);
 
         if (changed && propagate)
                 rd_kafka_metadata_cache_propagate_changes(rk);
 }
 
 
-/**
- * @brief Update the metadata cache with the provided metadata.
- *
- * @param abs_update int: absolute update: purge cache before updating.
- *
- * @locks rd_kafka_wrlock()
- */
-void rd_kafka_metadata_cache_update (rd_kafka_t *rk,
-                                     const rd_kafka_metadata_t *md,
-                                     int abs_update) {
-        struct rd_kafka_metadata_cache_entry *rkmce;
-        rd_ts_t now = rd_clock();
-        rd_ts_t ts_expires = now + (rk->rk_conf.metadata_max_age_ms * 1000);
-        int i;
-
-        rd_kafka_dbg(rk, METADATA, "METADATA",
-                     "%s of metadata cache with %d topic(s)",
-                     abs_update ? "Absolute update" : "Update",
-                     md->topic_cnt);
-
-        if (abs_update)
-                rd_kafka_metadata_cache_purge(rk, rd_false/*not observers*/);
-
-
-        for (i = 0 ; i < md->topic_cnt ; i++)
-                rd_kafka_metadata_cache_insert(rk, &md->topics[i], now,
-                                               ts_expires);
-
-        /* Update expiry timer */
-        if ((rkmce = TAILQ_FIRST(&rk->rk_metadata_cache.rkmc_expiry)))
-                rd_kafka_timer_start(&rk->rk_timers,
-                                     &rk->rk_metadata_cache.rkmc_expiry_tmr,
-                                     rkmce->rkmce_ts_expires - now,
-                                     rd_kafka_metadata_cache_evict_tmr_cb,
-                                     rk);
-
-        if (md->topic_cnt > 0 || abs_update)
-                rd_kafka_metadata_cache_propagate_changes(rk);
-}
-
-
 /**
  * @brief Remove cache hints for topics in \p topics
  *        This is done when the Metadata response has been parsed and
@@ -400,8 +464,8 @@ void rd_kafka_metadata_cache_update (rd_kafka_t *rk,
  *
  * @locks rd_kafka_wrlock()
  */
-void rd_kafka_metadata_cache_purge_hints (rd_kafka_t *rk,
-                                          const rd_list_t *topics) {
+void rd_kafka_metadata_cache_purge_hints(rd_kafka_t *rk,
+                                         const rd_list_t *topics) {
         const char *topic;
         int i;
         int cnt = 0;
@@ -409,19 +473,19 @@ void rd_kafka_metadata_cache_purge_hints (rd_kafka_t *rk,
         RD_LIST_FOREACH(topic, topics, i) {
                 struct rd_kafka_metadata_cache_entry *rkmce;
 
-                if (!(rkmce = rd_kafka_metadata_cache_find(rk, topic,
-                                                           0/*any*/)) ||
+                if (!(rkmce =
+                          rd_kafka_metadata_cache_find(rk, topic, 0 /*any*/)) ||
                     RD_KAFKA_METADATA_CACHE_VALID(rkmce))
                         continue;
 
-                rd_kafka_metadata_cache_delete(rk, rkmce, 1/*unlink avl*/);
+                rd_kafka_metadata_cache_delete(rk, rkmce, 1 /*unlink avl*/);
                 cnt++;
         }
 
         if (cnt > 0) {
                 rd_kafka_dbg(rk, METADATA, "METADATA",
-                             "Purged %d/%d cached topic hint(s)",
-                             cnt, rd_list_cnt(topics));
+                             "Purged %d/%d cached topic hint(s)", cnt,
+                             rd_list_cnt(topics));
                 rd_kafka_metadata_cache_propagate_changes(rk);
         }
 }
@@ -448,41 +512,42 @@ void rd_kafka_metadata_cache_purge_hints (rd_kafka_t *rk,
  *
  * @locks_required rd_kafka_wrlock()
  */
-int rd_kafka_metadata_cache_hint (rd_kafka_t *rk,
-                                  const rd_list_t *topics, rd_list_t *dst,
-                                  rd_kafka_resp_err_t err,
-                                  rd_bool_t replace) {
+int rd_kafka_metadata_cache_hint(rd_kafka_t *rk,
+                                 const rd_list_t *topics,
+                                 rd_list_t *dst,
+                                 rd_kafka_resp_err_t err,
+                                 rd_bool_t replace) {
         const char *topic;
-        rd_ts_t now = rd_clock();
+        rd_ts_t now        = rd_clock();
         rd_ts_t ts_expires = now + (rk->rk_conf.socket_timeout_ms * 1000);
         int i;
         int cnt = 0;
 
         RD_LIST_FOREACH(topic, topics, i) {
-                rd_kafka_metadata_topic_t mtopic = {
-                        .topic = (char *)topic,
-                        .err = err
-                };
+                rd_kafka_metadata_topic_t mtopic = {.topic = (char *)topic,
+                                                    .err   = err};
+                rd_kafka_metadata_topic_internal_t metadata_internal_topic =
+                    RD_ZERO_INIT;
                 /*const*/ struct rd_kafka_metadata_cache_entry *rkmce;
 
                 /* !replace: Dont overwrite valid entries */
-                if (!replace &&
-                    (rkmce =
-                     rd_kafka_metadata_cache_find(rk, topic, 0/*any*/))) {
+                if (!replace && (rkmce = rd_kafka_metadata_cache_find(
+                                     rk, topic, 0 /*any*/))) {
                         if (RD_KAFKA_METADATA_CACHE_VALID(rkmce) ||
                             (dst && rkmce->rkmce_mtopic.err !=
-                             RD_KAFKA_RESP_ERR__NOENT))
+                                        RD_KAFKA_RESP_ERR__NOENT))
                                 continue;
                         rkmce->rkmce_mtopic.err = err;
                         /* FALLTHRU */
                 }
 
-                rd_kafka_metadata_cache_insert(rk, &mtopic, now, ts_expires);
+                rd_kafka_metadata_cache_insert(rk, &mtopic,
+                                               &metadata_internal_topic, now,
+                                               ts_expires, rd_false, NULL, 0);
                 cnt++;
 
                 if (dst)
                         rd_list_add(dst, rd_strdup(topic));
-
         }
 
         if (cnt > 0)
@@ -500,21 +565,20 @@ int rd_kafka_metadata_cache_hint (rd_kafka_t *rk,
  *
  * @locks_acquired rd_kafka_wrlock()
  */
-int rd_kafka_metadata_cache_hint_rktparlist (
-        rd_kafka_t *rk,
-        const rd_kafka_topic_partition_list_t *rktparlist,
-        rd_list_t *dst,
-        int replace) {
+int rd_kafka_metadata_cache_hint_rktparlist(
+    rd_kafka_t *rk,
+    const rd_kafka_topic_partition_list_t *rktparlist,
+    rd_list_t *dst,
+    int replace) {
         rd_list_t topics;
         int r;
 
         rd_list_init(&topics, rktparlist->cnt, rd_free);
         rd_kafka_topic_partition_list_get_topic_names(rktparlist, &topics,
-                                                      0/*dont include regex*/);
+                                                      0 /*dont include regex*/);
         rd_kafka_wrlock(rk);
-        r = rd_kafka_metadata_cache_hint(rk, &topics, dst,
-                                         RD_KAFKA_RESP_ERR__WAIT_CACHE,
-                                         replace);
+        r = rd_kafka_metadata_cache_hint(
+            rk, &topics, dst, RD_KAFKA_RESP_ERR__WAIT_CACHE, replace);
         rd_kafka_wrunlock(rk);
 
         rd_list_destroy(&topics);
@@ -525,7 +589,7 @@ int rd_kafka_metadata_cache_hint_rktparlist (
 /**
  * @brief Cache entry comparator (on topic name)
  */
-static int rd_kafka_metadata_cache_entry_cmp (const void *_a, const void *_b) {
+static int rd_kafka_metadata_cache_entry_cmp(const void *_a, const void *_b) {
         const struct rd_kafka_metadata_cache_entry *a = _a, *b = _b;
         return strcmp(a->rkmce_mtopic.topic, b->rkmce_mtopic.topic);
 }
@@ -536,7 +600,7 @@ static int rd_kafka_metadata_cache_entry_cmp (const void *_a, const void *_b) {
  *
  * @locks rd_kafka_wrlock()
  */
-void rd_kafka_metadata_cache_init (rd_kafka_t *rk) {
+void rd_kafka_metadata_cache_init(rd_kafka_t *rk) {
         rd_avl_init(&rk->rk_metadata_cache.rkmc_avl,
                     rd_kafka_metadata_cache_entry_cmp, 0);
         TAILQ_INIT(&rk->rk_metadata_cache.rkmc_expiry);
@@ -552,11 +616,11 @@ void rd_kafka_metadata_cache_init (rd_kafka_t *rk) {
  *
  * @locks_required rd_kafka_wrlock()
  */
-void rd_kafka_metadata_cache_destroy (rd_kafka_t *rk) {
+void rd_kafka_metadata_cache_destroy(rd_kafka_t *rk) {
         rd_list_destroy(&rk->rk_metadata_cache.rkmc_observers);
         rd_kafka_timer_stop(&rk->rk_timers,
-                            &rk->rk_metadata_cache.rkmc_query_tmr, 1/*lock*/);
-        rd_kafka_metadata_cache_purge(rk, rd_true/*observers too*/);
+                            &rk->rk_metadata_cache.rkmc_query_tmr, 1 /*lock*/);
+        rd_kafka_metadata_cache_purge(rk, rd_true /*observers too*/);
         mtx_destroy(&rk->rk_metadata_cache.rkmc_full_lock);
         mtx_destroy(&rk->rk_metadata_cache.rkmc_cnd_lock);
         cnd_destroy(&rk->rk_metadata_cache.rkmc_cnd);
@@ -570,9 +634,9 @@ void rd_kafka_metadata_cache_destroy (rd_kafka_t *rk) {
  *
  * @locks_required rd_kafka_wrlock()
  */
-void
-rd_kafka_metadata_cache_wait_state_change_async (rd_kafka_t *rk,
-                                                 rd_kafka_enq_once_t *eonce) {
+void rd_kafka_metadata_cache_wait_state_change_async(
+    rd_kafka_t *rk,
+    rd_kafka_enq_once_t *eonce) {
         rd_kafka_enq_once_add_source(eonce, "wait metadata cache change");
         rd_list_add(&rk->rk_metadata_cache.rkmc_observers, eonce);
 }
@@ -585,21 +649,19 @@ rd_kafka_metadata_cache_wait_state_change_async (rd_kafka_t *rk,
  * @locks none
  * @locality any
  */
-int rd_kafka_metadata_cache_wait_change (rd_kafka_t *rk, int timeout_ms) {
+int rd_kafka_metadata_cache_wait_change(rd_kafka_t *rk, int timeout_ms) {
         int r;
 #if ENABLE_DEVEL
         rd_ts_t ts_start = rd_clock();
 #endif
         mtx_lock(&rk->rk_metadata_cache.rkmc_cnd_lock);
         r = cnd_timedwait_ms(&rk->rk_metadata_cache.rkmc_cnd,
-                             &rk->rk_metadata_cache.rkmc_cnd_lock,
-                             timeout_ms);
+                             &rk->rk_metadata_cache.rkmc_cnd_lock, timeout_ms);
         mtx_unlock(&rk->rk_metadata_cache.rkmc_cnd_lock);
 
 #if ENABLE_DEVEL
-        rd_kafka_dbg(rk, METADATA, "CACHEWAIT",
-                     "%s wait took %dms: %s",
-                     __FUNCTION__, (int)((rd_clock() - ts_start)/1000),
+        rd_kafka_dbg(rk, METADATA, "CACHEWAIT", "%s wait took %dms: %s",
+                     __FUNCTION__, (int)((rd_clock() - ts_start) / 1000),
                      r == thrd_success ? "succeeded" : "timed out");
 #endif
         return r == thrd_success;
@@ -611,8 +673,8 @@ int rd_kafka_metadata_cache_wait_change (rd_kafka_t *rk, int timeout_ms) {
  *        rd_kafka_metadata_cache_propagate_changes()
  */
 static int
-rd_kafka_metadata_cache_propagate_changes_trigger_eonce (void *elem,
-                                                         void *opaque) {
+rd_kafka_metadata_cache_propagate_changes_trigger_eonce(void *elem,
+                                                        void *opaque) {
         rd_kafka_enq_once_t *eonce = elem;
         rd_kafka_enq_once_trigger(eonce, RD_KAFKA_RESP_ERR_NO_ERROR,
                                   "wait metadata cache change");
@@ -627,7 +689,7 @@ rd_kafka_metadata_cache_propagate_changes_trigger_eonce (void *elem,
  * @locks_acquired rkmc_cnd_lock
  * @locality any
  */
-void rd_kafka_metadata_cache_propagate_changes (rd_kafka_t *rk) {
+void rd_kafka_metadata_cache_propagate_changes(rd_kafka_t *rk) {
         mtx_lock(&rk->rk_metadata_cache.rkmc_cnd_lock);
         cnd_broadcast(&rk->rk_metadata_cache.rkmc_cnd);
         mtx_unlock(&rk->rk_metadata_cache.rkmc_cnd_lock);
@@ -636,7 +698,6 @@ void rd_kafka_metadata_cache_propagate_changes (rd_kafka_t *rk) {
         rd_list_apply(&rk->rk_metadata_cache.rkmc_observers,
                       rd_kafka_metadata_cache_propagate_changes_trigger_eonce,
                       NULL);
-
 }
 
 /**
@@ -646,8 +707,9 @@ void rd_kafka_metadata_cache_propagate_changes (rd_kafka_t *rk) {
  * @locks rd_kafka_*lock()
  */
 const rd_kafka_metadata_topic_t *
-rd_kafka_metadata_cache_topic_get (rd_kafka_t *rk, const char *topic,
-                                   int valid) {
+rd_kafka_metadata_cache_topic_get(rd_kafka_t *rk,
+                                  const char *topic,
+                                  int valid) {
         struct rd_kafka_metadata_cache_entry *rkmce;
 
         if (!(rkmce = rd_kafka_metadata_cache_find(rk, topic, valid)))
@@ -658,7 +720,6 @@ rd_kafka_metadata_cache_topic_get (rd_kafka_t *rk, const char *topic,
 
 
 
-
 /**
  * @brief Looks up the shared metadata for a partition along with its topic.
  *
@@ -674,18 +735,20 @@ rd_kafka_metadata_cache_topic_get (rd_kafka_t *rk, const char *topic,
  *
  * @locks rd_kafka_*lock()
  */
-int rd_kafka_metadata_cache_topic_partition_get (
-        rd_kafka_t *rk,
-        const rd_kafka_metadata_topic_t **mtopicp,
-        const rd_kafka_metadata_partition_t **mpartp,
-        const char *topic, int32_t partition, int valid) {
+int rd_kafka_metadata_cache_topic_partition_get(
+    rd_kafka_t *rk,
+    const rd_kafka_metadata_topic_t **mtopicp,
+    const rd_kafka_metadata_partition_t **mpartp,
+    const char *topic,
+    int32_t partition,
+    int valid) {
 
         const rd_kafka_metadata_topic_t *mtopic;
         const rd_kafka_metadata_partition_t *mpart;
-        rd_kafka_metadata_partition_t skel = { .id = partition };
+        rd_kafka_metadata_partition_t skel = {.id = partition};
 
         *mtopicp = NULL;
-        *mpartp = NULL;
+        *mpartp  = NULL;
 
         if (!(mtopic = rd_kafka_metadata_cache_topic_get(rk, topic, valid)))
                 return -1;
@@ -696,8 +759,7 @@ int rd_kafka_metadata_cache_topic_partition_get (
                 return -1;
 
         /* Partitions array may be sparse so use bsearch lookup. */
-        mpart = bsearch(&skel, mtopic->partitions,
-                        mtopic->partition_cnt,
+        mpart = bsearch(&skel, mtopic->partitions, mtopic->partition_cnt,
                         sizeof(*mtopic->partitions),
                         rd_kafka_metadata_partition_id_cmp);
 
@@ -718,12 +780,12 @@ int rd_kafka_metadata_cache_topic_partition_get (
  *
  * @locks rd_kafka_*lock()
  */
-int rd_kafka_metadata_cache_topics_count_exists (rd_kafka_t *rk,
-                                                 const rd_list_t *topics,
-                                                 int *metadata_agep) {
+int rd_kafka_metadata_cache_topics_count_exists(rd_kafka_t *rk,
+                                                const rd_list_t *topics,
+                                                int *metadata_agep) {
         const char *topic;
         int i;
-        int cnt = 0;
+        int cnt     = 0;
         int max_age = -1;
 
         RD_LIST_FOREACH(topic, topics, i) {
@@ -731,10 +793,10 @@ int rd_kafka_metadata_cache_topics_count_exists (rd_kafka_t *rk,
                 int age;
 
                 if (!(rkmce = rd_kafka_metadata_cache_find(rk, topic,
-                                                           1/*valid only*/)))
+                                                           1 /*valid only*/)))
                         continue;
 
-                age = (int)((rd_clock() - rkmce->rkmce_ts_insert)/1000);
+                age = (int)((rd_clock() - rkmce->rkmce_ts_insert) / 1000);
                 if (age > max_age)
                         max_age = age;
                 cnt++;
@@ -743,7 +805,6 @@ int rd_kafka_metadata_cache_topics_count_exists (rd_kafka_t *rk,
         *metadata_agep = max_age;
 
         return cnt;
-
 }
 
 
@@ -756,8 +817,7 @@ int rd_kafka_metadata_cache_topics_count_exists (rd_kafka_t *rk,
  *
  * @locks_required rd_kafka_*lock()
  */
-int rd_kafka_metadata_cache_topics_to_list (rd_kafka_t *rk,
-                                             rd_list_t *topics) {
+int rd_kafka_metadata_cache_topics_to_list(rd_kafka_t *rk, rd_list_t *topics) {
         const struct rd_kafka_metadata_cache_entry *rkmce;
         int precnt = rd_list_cnt(topics);
 
@@ -782,26 +842,25 @@ int rd_kafka_metadata_cache_topics_to_list (rd_kafka_t *rk,
  *
  * @locks rd_kafka_*lock()
  */
-void rd_kafka_metadata_cache_dump (FILE *fp, rd_kafka_t *rk) {
+void rd_kafka_metadata_cache_dump(FILE *fp, rd_kafka_t *rk) {
         const struct rd_kafka_metadata_cache *rkmc = &rk->rk_metadata_cache;
         const struct rd_kafka_metadata_cache_entry *rkmce;
         rd_ts_t now = rd_clock();
 
-        fprintf(fp,
-                "Metadata cache with %d entries:\n",
-                rkmc->rkmc_cnt);
+        fprintf(fp, "Metadata cache with %d entries:\n", rkmc->rkmc_cnt);
         TAILQ_FOREACH(rkmce, &rkmc->rkmc_expiry, rkmce_link) {
                 fprintf(fp,
                         "  %s (inserted %dms ago, expires in %dms, "
                         "%d partition(s), %s)%s%s\n",
                         rkmce->rkmce_mtopic.topic,
-                        (int)((now - rkmce->rkmce_ts_insert)/1000),
-                        (int)((rkmce->rkmce_ts_expires - now)/1000),
+                        (int)((now - rkmce->rkmce_ts_insert) / 1000),
+                        (int)((rkmce->rkmce_ts_expires - now) / 1000),
                         rkmce->rkmce_mtopic.partition_cnt,
-                        RD_KAFKA_METADATA_CACHE_VALID(rkmce) ? "valid":"hint",
+                        RD_KAFKA_METADATA_CACHE_VALID(rkmce) ? "valid" : "hint",
                         rkmce->rkmce_mtopic.err ? " error: " : "",
-                        rkmce->rkmce_mtopic.err ?
-                        rd_kafka_err2str(rkmce->rkmce_mtopic.err) : "");
+                        rkmce->rkmce_mtopic.err
+                            ? rd_kafka_err2str(rkmce->rkmce_mtopic.err)
+                            : "");
         }
 }
 
