@@ -1,7 +1,8 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2019 Magnus Edenhill
+ * Copyright (c) 2019-2022, Magnus Edenhill
+ *               2023, Confluent Inc.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -217,6 +218,7 @@ typedef struct rd_kafka_mock_msgset_s {
         TAILQ_ENTRY(rd_kafka_mock_msgset_s) link;
         int64_t first_offset; /**< First offset in batch */
         int64_t last_offset;  /**< Last offset in batch */
+        int32_t leader_epoch; /**< Msgset leader epoch */
         rd_kafkap_bytes_t bytes;
         /* Space for bytes.data is allocated after the msgset_t */
 } rd_kafka_mock_msgset_t;
@@ -234,6 +236,8 @@ typedef struct rd_kafka_mock_committed_offset_s {
 } rd_kafka_mock_committed_offset_t;
 
 
+TAILQ_HEAD(rd_kafka_mock_msgset_tailq_s, rd_kafka_mock_msgset_s);
+
 /**
  * @struct Mock partition
  */
@@ -241,6 +245,8 @@ typedef struct rd_kafka_mock_partition_s {
         TAILQ_ENTRY(rd_kafka_mock_partition_s) leader_link;
         int32_t id;
 
+        int32_t leader_epoch;          /**< Leader epoch, bumped on each
+                                        *   partition leader change. */
         int64_t start_offset;          /**< Actual/leader start offset */
         int64_t end_offset;            /**< Actual/leader end offset */
         int64_t follower_start_offset; /**< Follower's start offset */
@@ -252,7 +258,7 @@ typedef struct rd_kafka_mock_partition_s {
                                                  *   in synch with end_offset
                                                  */
 
-        TAILQ_HEAD(, rd_kafka_mock_msgset_s) msgsets;
+        struct rd_kafka_mock_msgset_tailq_s msgsets;
         size_t size;     /**< Total size of all .msgsets */
         size_t cnt;      /**< Total count of .msgsets */
         size_t max_size; /**< Maximum size of all .msgsets, may be overshot. */
@@ -447,6 +453,14 @@ rd_kafka_mock_partition_log_append(rd_kafka_mock_partition_t *mpart,
                                    const rd_kafkap_str_t *TransactionalId,
                                    int64_t *BaseOffset);
 
+rd_kafka_resp_err_t rd_kafka_mock_partition_leader_epoch_check(
+    const rd_kafka_mock_partition_t *mpart,
+    int32_t leader_epoch);
+
+int64_t rd_kafka_mock_partition_offset_for_leader_epoch(
+    const rd_kafka_mock_partition_t *mpart,
+    int32_t leader_epoch);
+
 
 /**
  * @returns true if the ApiVersion is supported, else false.
@@ -492,6 +506,7 @@ rd_kafka_mock_cgrp_member_add(rd_kafka_mock_cgrp_t *mcgrp,
                               rd_kafka_mock_connection_t *mconn,
                               rd_kafka_buf_t *resp,
                               const rd_kafkap_str_t *MemberId,
+                              const rd_kafkap_str_t *GroupInstanceId,
                               const rd_kafkap_str_t *ProtocolType,
                               rd_kafka_mock_cgrp_proto_t *protos,
                               int proto_cnt,
