@@ -1,7 +1,7 @@
 /*
  * librdkafka - Apache Kafka C library
  *
- * Copyright (c) 2019, Magnus Edenhill
+ * Copyright (c) 2019-2022, Magnus Edenhill
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -304,7 +304,7 @@ void do_test_consumer_producer_txn(void) {
         const char *c1_groupid = input_topic;
         const char *c2_groupid = output_topic;
         rd_kafka_t *p1, *p2, *c1, *c2;
-        rd_kafka_conf_t *conf, *tmpconf, *c1_conf;
+        rd_kafka_conf_t *conf, *tmpconf;
         uint64_t testid;
 #define _MSGCNT (10 * 30)
         const int txncnt = 10;
@@ -364,8 +364,7 @@ void do_test_consumer_producer_txn(void) {
         test_conf_set(tmpconf, "isolation.level", "read_committed");
         test_conf_set(tmpconf, "auto.offset.reset", "earliest");
         test_conf_set(tmpconf, "enable.auto.commit", "false");
-        c1_conf = rd_kafka_conf_dup(tmpconf);
-        c1      = test_create_consumer(c1_groupid, NULL, tmpconf, NULL);
+        c1 = test_create_consumer(c1_groupid, NULL, tmpconf, NULL);
         test_consumer_subscribe(c1, input_topic);
 
         /* Create Producer 2 */
@@ -382,8 +381,6 @@ void do_test_consumer_producer_txn(void) {
         c2 = test_create_consumer(c2_groupid, NULL, tmpconf, NULL);
         test_consumer_subscribe(c2, output_topic);
 
-        rd_kafka_conf_destroy(conf);
-
         /* Keep track of what messages to expect on the output topic */
         test_msgver_init(&expect_mv, testid);
 
@@ -391,9 +388,11 @@ void do_test_consumer_producer_txn(void) {
                 int msgcnt2 = 10 * (1 + (txn % 3));
                 rd_kafka_message_t *msgs[_MSGCNT];
                 int i;
-                rd_bool_t do_abort          = !(txn % 3);
-                rd_bool_t recreate_consumer = do_abort && txn == 3;
-                rd_kafka_topic_partition_list_t *offsets;
+                rd_bool_t do_abort = !(txn % 3);
+                rd_bool_t recreate_consumer =
+                    (do_abort && txn == 3) || (!do_abort && txn == 2);
+                rd_kafka_topic_partition_list_t *offsets,
+                    *expected_offsets = NULL;
                 rd_kafka_resp_err_t err;
                 rd_kafka_consumer_group_metadata_t *c1_cgmetadata;
                 int remains = msgcnt2;
@@ -452,6 +451,28 @@ void do_test_consumer_producer_txn(void) {
                 TEST_CALL_ERROR__(rd_kafka_send_offsets_to_transaction(
                     p2, offsets, c1_cgmetadata, -1));
 
+                if (recreate_consumer && !do_abort) {
+                        expected_offsets =
+                            rd_kafka_topic_partition_list_new(offsets->cnt);
+
+                        /* Cannot use rd_kafka_topic_partition_list_copy
+                         * as it needs to be destroyed before closing the
+                         * consumer, because of the _private field holding
+                         * a reference to the internal toppar */
+                        for (i = 0; i < offsets->cnt; i++) {
+                                rd_kafka_topic_partition_t *rktpar =
+                                    &offsets->elems[i];
+                                rd_kafka_topic_partition_t *rktpar_new;
+                                rktpar_new = rd_kafka_topic_partition_list_add(
+                                    expected_offsets, rktpar->topic,
+                                    rktpar->partition);
+                                rktpar_new->offset = rktpar->offset;
+                                rd_kafka_topic_partition_set_leader_epoch(
+                                    rktpar_new,
+                                    rd_kafka_topic_partition_get_leader_epoch(
+                                        rktpar));
+                        }
+                }
 
                 rd_kafka_consumer_group_metadata_destroy(c1_cgmetadata);
 
@@ -481,12 +502,57 @@ void do_test_consumer_producer_txn(void) {
                         rd_kafka_consumer_close(c1);
                         rd_kafka_destroy(c1);
 
-                        c1 = test_create_consumer(c1_groupid, NULL, c1_conf,
+                        tmpconf = rd_kafka_conf_dup(conf);
+                        test_conf_set(tmpconf, "isolation.level",
+                                      "read_committed");
+                        test_conf_set(tmpconf, "auto.offset.reset", "earliest");
+                        test_conf_set(tmpconf, "enable.auto.commit", "false");
+                        c1 = test_create_consumer(c1_groupid, NULL, tmpconf,
                                                   NULL);
                         test_consumer_subscribe(c1, input_topic);
+
+
+                        if (expected_offsets) {
+                                rd_kafka_topic_partition_list_t
+                                    *committed_offsets =
+                                        rd_kafka_topic_partition_list_copy(
+                                            expected_offsets);
+                                /* Set committed offsets and epochs to a
+                                 * different value before requesting them. */
+                                for (i = 0; i < committed_offsets->cnt; i++) {
+                                        rd_kafka_topic_partition_t *rktpar =
+                                            &committed_offsets->elems[i];
+                                        rktpar->offset = -100;
+                                        rd_kafka_topic_partition_set_leader_epoch(
+                                            rktpar, -100);
+                                }
+
+                                TEST_CALL_ERR__(rd_kafka_committed(
+                                    c1, committed_offsets, -1));
+
+                                if (test_partition_list_and_offsets_cmp(
+                                        expected_offsets, committed_offsets)) {
+                                        TEST_SAY("expected list:\n");
+                                        test_print_partition_list(
+                                            expected_offsets);
+                                        TEST_SAY("committed() list:\n");
+                                        test_print_partition_list(
+                                            committed_offsets);
+                                        TEST_FAIL(
+                                            "committed offsets don't match");
+                                }
+
+                                rd_kafka_topic_partition_list_destroy(
+                                    committed_offsets);
+
+                                rd_kafka_topic_partition_list_destroy(
+                                    expected_offsets);
+                        }
                 }
         }
 
+        rd_kafka_conf_destroy(conf);
+
         test_msgver_init(&actual_mv, testid);
 
         test_consumer_poll("Verify output topic", c2, testid, -1, 0,
